/***************************************************************************
                          databaseaccess.h  -  description
                             -------------------
    begin                : Fr Nov 5 2004
    copyright            : (C) 2004 by Marcel Mulch
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DATABASEACCESS_H
#define DATABASEACCESS_H


// dump flags
#define HOBIT_DUMP_FLAGS_TRANSACTIONS   0x00010000
#define HOBIT_DUMP_FLAGS_BALANCE        0x00020000
#define HOBIT_DUMP_FLAGS_USERS          0x00040000
#define HOBIT_DUMP_FLAGS_ACCOUNT_HEADER 0x00080000
#define HOBIT_DUMP_FLAGS_STANDINGORDERS 0x00100000
#define HOBIT_DUMP_FLAGS_DATEDXFERS     0x00200000
#define HOBIT_DUMP_FLAGS_NOTEDXFERS     0x00400000

#define HOBIT_DUMP_FLAGS_DEFAULT \
  HOBIT_DUMP_FLAGS_ACCOUNT_HEADER | \
  AQM_TRANSACTION_MATCH_DEFAULT


  /* Internationalization */
#ifdef ENABLE_NLS
# include <libintl.h>
# include <locale.h>
# define I18N(m) gettext(m)
#else
# define I18N(m) m
#endif
  

#include <openhbci2.h>
using namespace HBCI;

#include <qobject.h> //for translation system

#include "aqmoney2/aqmapi.h"
#include "aqmoney2/account.h"
#include "hobit_string.h"
#include "transaction.h"
#include "standingorder.h"
#include "datedtransfer.h"
#include "hobit_bank.h"
#include "hobit_transfertemplate.h"

#include <list>

#include <qdatetime.h>
using namespace std;


/**                                                                  
  *@author Marcel Mulch
  */

class DatabaseAccess: public QObject{
Q_OBJECT //neccessary for translations
public: 
	DatabaseAccess(Pointer <AqMoneyAPI> api);
	virtual ~DatabaseAccess(); 

  /**  This method creates a new User configuration. Like most of the code
  *  hobit consists of this method was also inspired by AqMoney.
  *
  * @author Marcel Mulch
  * @date 20.10.2004
  *
  * @param mediumname  the name of the security medium. this might be the
  *                    path to the keyfile if you are using an rsa-medium
  *
  * @param mediumtype  the type of the medium. OpenHBCI (and hbci in general)
  *                    differs between 2 types of security mediums. The
  *                    first one is RSA-File. The other one is DDV-Chipcard
  *                    With this parameter you can tell HoBIT (and OpenHBCI)
  *                    which type of medium you are using. <br> <br>
  *
  *                    If you are using RDH-File:   "RDHFile" <br>
  *                    If you are using DDV-Card:   "???"     <br>
  *
  * @param readonly    You can decide if the security medium should be set to
  *                    readonly. If you want so set readonly to true
  *
  * @param country     This is the countrycode. It is needed to tell HBCI the
  *                    location of your bank. This parameter should be 280 if
  *                    you are trying to connect to a german bank.
  *
  * @param instid      The institute code. Every bank has a special code, what
  *                    is unique and identifies it. In Germany this bank code
  *                    is just called Bankleitzahl.
  *
  * @param userid      In HBCI users have an Identification code too. Because
  *                    Users get this code from their bank I do not have to
  *                    explain it. Look into the HBCI-Specification if unsure.
  *
  * @param customerid  In HBCI customers have an Identification code too. Because
  *                    Customers get this code from their bank I do not have to
  *                    explain it. Look into the HBCI-Specification if unsure
  *                    In most cases the customerid will be equal to the user id.
  *
  *
  * @param server      this might be the url or the ip-adress of the bank server
  */
virtual  Error  createUser(const string &mediumname,
                           const string &mediumType,
                           bool readonly,
                           int country,
                           const string &instid,
                           int hbciVersion,
                           const string &userid,
                           const string &customerid,
                           const string &server);

/** Reads the data from the chipcard. There MAY be userid, bankid, server. Returns the number of valid user data sets.
*   If no valid set is found, strings in set 1 (the first record) are used. Else the data of the last valid set are used.
*/
 int readCardData(int & country, QString & bankId, QString & userId, QString & server, QString & cardnumber);
                           
/** creates a new account with the overgiven values
  * this procedure will for example be done, when 
  * the retrieving of the account list from the 
  * bank server fails 
  *
  * @param name: Name of the account owner
  * @param accountId: the account number
  * @param accountSubId: Hbci allows subids for accounts 
  * 			 this parameter is normaly not needed
  * @param accountName:  you can give your account an description
  *
  */			   
virtual HobitError addAccount(const string& name = "", 
			      const string &accountId = "", 
			      const string &accountSubId = "", 
			      const string& accountName = "");
			      
			      
//** deletes an existing account */
//virtual HobitError deleteAccount(const string &accountId = "");
			      


/** Returns a std::list which includes all registered users as a struct
*/
  virtual  list<Pointer<User> > listUsers();

/** removes all existing users from the gwen database
*/                                                                                                                  
 virtual  void deleteAllBanks();


/** Method to change used hbci version
*   checks if an overhanded hbci version is supported
*   by the bank and returns false if not
*/
  virtual  bool changeHbciVersion(int version);


/** This method generates the customers ini-letter
*/
  virtual string generateIniLetter(int country, const string& bankcode, const string& customerId);


 /** lists all transactions of an account which were made during
 *  a specific time periode */
  virtual list<Pointer <Transaction> >  listTransactions(int country,
                                                             const string &bankCode,
                                                             const string &accountId,
                                                             const string &accountSubId,
                                                             const Date &fromDate,
                                                             const Date &toDate);

 /** lists all transactions of an account which were made during
 *  a specific time periode */
  virtual list <Pointer <DatedTransfer> >  listDatedTransfers(int country,
                                                              const string &bankCode,
                                                              const string &accountId,
                                                              const string &accountSubId);

  /** lists all transactions of an account which were made during
 *  a specific time periode */
  virtual list <Pointer <StandingOrder> >  listStandingOrders(int country,
                                                              const string &bankCode,
                                                              const string &accountId,
                                                              const string &accountSubId);


/** Returns the date of the last stored transaction.
  * This is for example needed for updating
  * transactions list, but can also be used for gui stuff
  * !!!!WARNING: This date may be in the future!!!!
  */
  virtual QDate getLastDateOfTransactions();
  
  
/**
* Method that reads the ktoblzcheck-banklist and returns 
* a list of HoBIT_Bank-Objects
*
* I was heavily inspired by the ktoblzcheck-source, when I 
* wrote this method
*
* @author Narcel Mulch
* @date 4.12.2004
*
*/
list<HoBIT_Bank> readBankFile(const string &filename);

/**
* Method that reads out the ZKA -banklist fints_institute.csv and returns 
* a list of HoBIT_Bank-Objects
*
*/
bool writeHbdBankFile(const string &filename, list<HoBIT_Bank> & bankList); 

 /** returns the hash fingerprint of the institute after receiving the bank servers public keys 
 */
  string getBankFingerPrint();

  
 /** returns the fingerprint of the current public key
 */
  string getMyOwnFingerPrint();
  
  
 /** changes the account description */
  void changeAccountName(const string& accid, const string& name);
  
  
 
  /** writes all templates to file */
  Error saveTransferTemplates(const string& templateFile);
  
  /** reads stored templates from file */
  Error readTransferTemplates(const string& templateFile);

  
  /** adding transfer template */
  void addTransferTemplate(Pointer<HoBIT_TransferTemplate> temp);
  
  
  /** returns all transfer templates */
  list<Pointer<HoBIT_TransferTemplate> > getTransferTemplates(){
  return _templates;
  }
  
  
  /** deletes an transfer template */
  void deleteTransferTemplate(const string& ownAccountId,
  			      const string& otherName, 
			      const string& otherAccountId,
			      const string& otherBank,
			      double value,
			      const QStringList& purpose);
  
  
  
  
private:
  Pointer<AqMoneyAPI> _api;  
  list<Pointer<HoBIT_TransferTemplate> > _templates;

};

#endif
