/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

 #include "globals.h"

 #include <qdir.h>
 #include <qmessagebox.h>
 #include <qtextcodec.h>
 #include <qapplication.h>

//static variable for MP_DIR()
QString  mp_dir;

/** set the main project directory from the program filepath
*/
void setMP_DIR(QString programFilePath){
 if(mp_dir.length() == 0){ // do not allow modifications
    qDebug("Program file path : %s", programFilePath.latin1());
    QFileInfo mpBinary(programFilePath);
    QDir dir( mpBinary.dir(true)); //absolute directory path of the binary, we are in moneypenny/bin now
    dir.cd("..");// go up to the main dir. This is ~/moneypenny, when running from the development tree
                 // When running from the install tree it is /usr
    mp_dir = dir.path() +"/";
    if(QFile(mp_dir + "moneypenny.pro").exists())
      qDebug("Running from development directory tree.");
    else
     if(mp_dir.find("usr")>=1){ // may be /usr or /usr/local or /UNIONFS/usr on a KNOPPIX CD
       mp_dir = mp_dir + "share/moneypenny/";
     }  
    QString fileName = mp_dir + MONEYPENNY_DATA_DIR + "/" + MONEYPENNY_HBCI_BANKDATA_FILE;
    if(!QFile(fileName).exists()){
     QString msg("Error in installation. Can not find file ");
     msg += fileName + QString(" in moneypenny's main directory.");
     qDebug(msg.latin1());
     qDebug("MoneyPenny (false) main directory MP_DIR: %s", mp_dir.latin1());
     exit (4);
    }
 }
 qDebug("MoneyPenny main directory MP_DIR: %s", mp_dir.latin1());
}

/** Returns top resource dir with trailing /
*/
QString MP_DIR(){
  return mp_dir;
}

/** Check for existing .qm file.
*/
QString tryLocale(QString language, QString baseName){
    QString qmFileName =  MP_DIR() + QString(MONEYPENNY_TRANSLATIONS_DIR) + "/" + baseName + language + ".qm";
    QFile qmFile(qmFileName);
    if(!qmFile.exists()){
       qDebug("Could not find translation file: %s", qmFileName.latin1());
       return "";
    }
    return qmFileName;
}

/** Returns the name of the translation file for the local language. Returns
  * an empty string, if it does nor exist
  */
QString QM_FILE_NAME(QString baseName){
    QString result;
    QString language = QTextCodec::locale();
    qDebug("System language setting (locale): %s", language.latin1());
    result = tryLocale(language, baseName);
    if(result.isEmpty()){
      language = language.left(language.find(".")); //remove encoding info e.g. .UTF8
      result = tryLocale(language, baseName);
    }
    if(result.isEmpty()){
      language = language.left(language.find("_")); //remove sublanguage e.g. _eur and try again
      result = tryLocale(language, baseName);
    }
    return result;
 }

 QFont calculateScreenFont(){
      //Set default font size according to screen and teststring size
  uint globalFontSize = 18;
  uint screenWidth =  qApp->desktop()->width();
  uint screenHeight = qApp->desktop()->height();
  uint charsPerLine = (110 * screenWidth) / 1024; //use more chars on a bigger screen
  if (charsPerLine > 140) //more makes no sense
    charsPerLine = 140;
  qDebug("Screensize: %i * %i  Chars per line: %i", screenWidth, screenHeight, charsPerLine);
  if ((double)screenWidth / (double)screenHeight > (4.0 / 3.0)) 
    charsPerLine = (charsPerLine * 3 * 16) / 9 /4; //longer lines on a wide-screen
  if (screenWidth >1600)
    screenWidth = 1600; //not more than 1600, dialogs do not grow
  uint stringWidth; 
  QFont globalFont(MONEYPENNY_SCREEN_FONT, globalFontSize);  //The FreeSans font familiy must be available in /usr/X11R6/lib/X11/fonts/truetype
                                                // See knx2moneypenny.sxw for details.
  QString testString(QObject::tr("abcdefghijklmnopqrstuvwxyz    eeeerrmnncy.,!?1234567890", 
                         "String with the alphabet for calculating font size. Take care of character probabilty. Space is a frequent character!"));
  do{  
    globalFontSize--; 
    QFontMetrics metrics(globalFont);
    stringWidth = metrics.width(testString);
    globalFont.setPointSize(globalFontSize); 
    qDebug("Testing font size: %i", globalFontSize);
   }while ((stringWidth * charsPerLine) / testString.length() > screenWidth ); 
   qDebug("Using font : %s   Fontsize: %i",globalFont.rawName().latin1(),globalFontSize);
  return globalFont;  
 }   
