/***************************************************************************
                          guiiniletter.h  -  description
                             -------------------
    begin                : So Nov 7 2004
    copyright            : (C) 2004 by Marcel Mulch
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GUIINILETTER_H
#define GUIINILETTER_H

#include <qwidget.h>
#include <qcanvas.h>
#include <qprinter.h>
#include <qwidgetstack.h>

#include <guiiniletterbase.h>
#include <hobit.h>

class GuiMain;



/**
*This is the base dialog for showing and printing the ini-letter
*/
class GuiIniLetter : public GuiIniLetterBase
{
  Q_OBJECT

public:

  /**
  *constructor of GuiIniLetter
  *
  *@param parent a pointer on the main widgets widget stack
  *@param name widgets title
  *@param hobit the hobit object, needed for hbci functions
  */
  GuiIniLetter( QWidget *parent, GuiMain* guimain, const char *name=0, HoBIT* hobit = 0);

  /**
  *destructor of GuiIniLetter
  */
  ~GuiIniLetter();

  /**
  *Generates an new ini-Letter and shows it on the textedit.
  */
  void newIniLetter();

  /**
  *Renews the layout and sets up the _View according to preview flag.
  *@param re pointer on the QResizeEvent - see in QT-Api for more help
  */
  void resizeEvent(QResizeEvent * re);

/** set hobit object*/
  void setHoBIT(HoBIT* hobit){
   _hobit = hobit;
  }

signals:

  /**
  *To be connected to GuiMain::slotAccountLogin()
  *
  */
  void terminated();
  
  /** Emitted on Add Printer
  */
  void addPrinterRequest();

private slots:

  /**
  * Opens the printer-dialog and starts printing
  * the ini-letter
  */
  void printIniLetter();

  /**
  *Calls the printer setup dialog.
  */
  void slotSetupPrinter();

  /**
  *Displays the text on the canvas, already scaled for printing.
  */
  void slotPreview();

  /**
  *Fires signal cacelled.
  */
  void slotCancel();

  /**
  *Shows the iniletter text scaled 1:1.
  */

  void showReadableText();
  /**
  *Shows a scaled print preview.
  */
  void showPreviewText();
  
  /**
  *  Hardware Setup
  */
  void slotHardware();

private:

  /**
  *function for creating the Layout
  */
  virtual void doLayout();

private:
  HoBIT *_hobit;
  GuiMain* _main;
  QWidgetStack *_stack;
  QCanvas *_Canvas;
  QCanvasView *_View;
  QCanvasText *_Text;
  QFont * _Font;
  QPrinter *_Printer;

  /**
  *Indicates, that the user has setup the printer
  */
  bool _SetupOk;

  /**
  *Flag for viewing state
  */
  bool _ShowPreview;

  /**
  *left border on printed page in cm
  */
  double _LeftBorder;

  /**
  *Top border on printed page in cm
  */
  double _TopBorder;
};

#endif
