/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "guitransferform.h"
#include "hobit_transfertemplate.h"
#include "dlgconfirmtransaction.h"

#include "globals.h"
#include <qlineedit.h>
#include <qcombobox.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qframe.h>
#include <qlayout.h>
#include <qwidget.h>
#include <qlineedit.h>
#include <qdatetimeedit.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qvalidator.h>
#include <qapplication.h>
#include <list>
#include <string>
using namespace std;



/**
*constructor of GuiTransferForm
*
*@param main a pointer on moneypenny´s main class
*@param parent a pointer on the main widgets widget stack
*@param name widgets title
*@param hobit the hobit object, needed for hbci functions
*/
GuiTransferForm::GuiTransferForm(QWidget *parent, const char *name, HoBIT* hobit )
:GuiTransferFormBase(parent,name) {
 _hobit = hobit;
 _ReconfirmationHeader = tr("You are about to create the following transfer: ");
 _TemplateCreated = false;
 refresh();
 _banks = _hobit->listOfAllKnownBanks();
 QRegExp rx("[0-9]{1,9},{1,1}[0-9]{1,2}"); // one to seven digits, a comma and one or two digits
 lineEditValue->setValidator(new QRegExpValidator(rx, lineEditValue, "ValueValidator" ));
 
 // if a lineEdit get changed -> call function for validation
 connect( lineEditOtherName, SIGNAL( textChanged(const QString&) ), this, SLOT( validateInputOtherName()) );
 connect( lineEditOtherAccountId, SIGNAL( textChanged(const QString&) ), this, SLOT( validateInputOtherAccountId()) );
 connect( lineEditOtherBankCode, SIGNAL( textChanged(const QString&) ), this, SLOT( validateInputOtherBankCode()) );
// connect( lineEditValue, SIGNAL( textChanged(const QString&) ), this, SLOT( validateInputValue()) );
 connect( lineEditPurpose, SIGNAL( textChanged(const QString&) ), this, SLOT( validateInputPurpose()) );
 connect( lineEditPurpose2, SIGNAL( textChanged(const QString&) ), this, SLOT( validateInputPurpose2()) );

 //initialize purpose lines with empty strings
 _PurposeLines = new QValueVector<QString>();
 _PurposeLines->resize(scrollBarLines->maxValue() + 2);
 _IsScrolling = false;
 // Clear Form
 clear();
 _FormType = SINGLE_TRANSFER_WIDGET; 
}

/**
*destructor of GuiTransferForm
*/
GuiTransferForm::~GuiTransferForm(){
  if(_PurposeLines)
    delete _PurposeLines;
  qDebug("Destructor GuiTransferForm finished");
}

/** 
*called function after pressing button of formular,
*gets current inputs of formular for transferjob  
*/
void GuiTransferForm::accept(){
 if(!isOk())
   return;
  
  int command = userReconfirmation();
  if ( command == DlgConfirmTransaction::Reject) 
    return;   
           
  _OurAccount = _hobit->getCurrentAccount();    //DO NOT strip any leading zeros here!   
  _OtherName = lineEditOtherName->text();
  _OtherBankCode = HoBIT_String::toIdFormat(lineEditOtherBankCode->text());
  _OtherAccountId = HoBIT_String::toIdFormat(lineEditOtherAccountId->text());
  QString value = lineEditValue->text();                  
  _Value = Value(value.ascii(), "EUR");
    
  if (createJob()){
   
    switch (command){
     case DlgConfirmTransaction::SendLater: 
          {
            QMessageBox::information(this,tr("Information"), 
             tr("The job was added to the job queue."));
            break;
           }      
     case DlgConfirmTransaction::SendNow: 
           {
             emit sendNow();
             break;      
           }  
     case DlgConfirmTransaction::SendNowAndGetTransactions: 
           {
              _hobit->createJobGetBalance();
              _hobit->createJobGetTransactions(Date(), Date(), "*");
              emit sendNow();
              break;
           }  
     default: return; //emergency break
    } 
  }
  else
   QMessageBox::critical(this, tr("Error"), tr("Job creation failed.") + "\n" +
     tr("Please refer to log file for details and send a bug report."));   
}

void GuiTransferForm::saveTemplate(){
  HoBIT_TransferTemplate * transferTemplate;
  
  QString ourAccountId = _hobit->getCurrentAccount();
  QString otherName = lineEditOtherName->text();
  QString otherBankCode = lineEditOtherBankCode->text();
  QString otherAccountId = lineEditOtherAccountId->text();
  double value = 0.0;  //force zero! 
  
  updatePurposeList();
  transferTemplate = 
     new HoBIT_TransferTemplate(ourAccountId,
                                otherName,
                                otherAccountId,
                                otherBankCode,
                                value,
                                _PurposeList);
   if(transferTemplate->isValid()) {                            
     _hobit->addTransferTemplate(transferTemplate);
     _TemplateCreated = true;
     emit refreshRequest();
     QMessageBox::information(this, tr("Information"), tr("This transfer has been added to the list of transfer templates."));
  }
  else {
    delete transferTemplate;
    QMessageBox::critical(this, tr("Error"), tr("Please fill in all fields. Value and purpose fields may be empty."));
  }
}

/** restores a template */
void GuiTransferForm::fromTemplate(){
  emit templateRequest(_FormType); //send this to adjust the radio buttons in template form
  emit raiseWidget(TRANSFER_TEMPLATES_WIDGET);
}


/**
*Set Current used Account to Label in Formular 
*/
void GuiTransferForm::refresh()
{
    if(_TemplateCreated){ //DO NOT update, if refresh request is due do template saving
        _TemplateCreated= false;
        return;
    }    
    textLabelAccount->setText(_hobit->getCurrentAccount());
    
    Pointer<Account> account;
    
    if (_hobit->getCurrentAccount() != "") {
    account = _hobit->findAccount(_hobit->getCurrentAccount().ascii());
    
    if (account.isValid()){
         textLabelName->setText(account.ref().userName().c_str());
    }
    else textLabelName->setText("No username found");
    }

    QDate curdate; curdate = curdate.currentDate();
    dateEditFirst->setDate(curdate);

    dateEditLast->setDate(QDate(curdate.year(),12,31));

    spinBoxDay->setMinValue(1);
    spinBoxDay->setMaxValue(30);
}

/** 
*validation of lineEdit name of receiver, 
*opens messagebox if validation is false
*/
void GuiTransferForm::validateInputOtherName(){ 
 
 // set input of lineEdit to var
 QString otherName = HoBIT_String::replaceSpecialChars(lineEditOtherName->text());
// QString otherName = lineEditOtherName->text();
 // regular expression allows only letters and blanks with a length of 0 to 35
 QString exp = "[a-z,A-Z,ä,ö,ü,Ä,ö,Ü,ß,\\-,\\+,\\s\\,\\.]{0,27}";  //regex seems to be buggy with respect to Umlaute
 QRegExp rx(exp);
 QRegExpValidator v( rx, 0 );
 int pos = 0;
 // if validation is true current input get saved to var. var is used for check next time the text get changed
 if(v.validate(otherName, pos)==QValidator::Acceptable  || otherName.isEmpty()) _LastInputOtherName = otherName;
 // if validation is false messagebox get opened for show that an invalid input was made
 else{
   QMessageBox::warning( this,
                         tr("Error"),
                         tr("Input Error\n Only letters are allowed here. Not more than 27 letters"),
                         tr("OK"), 
                         QString::null, 0, 1 ) ;
                         }
  updateEditor(lineEditOtherName, _LastInputOtherName);
 }

/** 
*validation of lineEdit accountId, 
*opens messagebox if validation is false
*/
void GuiTransferForm::validateInputOtherAccountId()
{
   // set input of lineEdit to var
   QString otherAccountId = lineEditOtherAccountId->text();
  
   // if validation is true current input get saved to var. var is used for check next time the text get changed
   if(_hobit-> validateAccountIdString(otherAccountId)) 
     _LastInputOtherAccountId = otherAccountId;
   // if validation is false messagebox get opened for show that an invalid input was made
   else { 
     lineEditOtherAccountId->setText(_LastInputOtherAccountId);
     QMessageBox::warning( this,
                         tr("Error"),
                         tr("Input Error\n Only digits are allowed here.\nMaximum is 15 digits."),
                         tr("OK"), 
                         QString::null, 0, 1 ) ;
   }
 }

/** 
*validation of lineEdit bankCode, 
*opens messagebox if validation is false 
*/
void GuiTransferForm::validateInputOtherBankCode()
{
  // set input of lineEdit to var
  QString otherBankCode = lineEditOtherBankCode->text();
   // regular expression allows only digits with a length of 0 to 15
  QRegExp rx( "[0-9]{0,8}" );
  QRegExpValidator v( rx, 0 );
  int pos = 0;
 
  // if validation is true current input get saved to var. var is used for check next time the text get changed
  if(v.validate(otherBankCode, pos)==QValidator::Acceptable  || otherBankCode.isEmpty()) _LastInputOtherBankCode = otherBankCode;
  // if validation is false messagebox get opened for show that an invalid input was made
  else { lineEditOtherBankCode->setText(_LastInputOtherBankCode);
  QMessageBox::warning( this,
                       tr("Error"),
                       tr("Input Error\n Only digits are allowed here. Exactly 8 digits required."),
                       tr("OK"), 
                       QString::null, 0, 1 ) ;
                       }
 
  //we are ready to check for a bank
  if(otherBankCode.length() == 8){
  int nBank = 0;
    list<HoBIT_Bank>::iterator bit;
    QString bankName = tr("Could not find a bank with the code you input.");
    if (!_banks.empty()){
      bool found = false;
      for (bit = _banks.begin(); (bit != _banks.end()) && (! found); bit++)
      {
        nBank++;
        HoBIT_Bank & bank =*bit;
        if(bank.getBankId()== otherBankCode){
          bankName = bank.getBankName();
          found = true; // bank found, stop it
        }
      }// end for
    }// end if ! empty
    textLabelBankName->setText(bankName);
  }//end if length == 8
  else textLabelBankName->setText("");
}



/** 
*validation of lineEdit Purpose1, 
*opens messagebox if validation is false
*/
void GuiTransferForm::validateInputPurpose()
{
 if(_IsScrolling) //No validation when scrolling, otherwise an endless loop is created
   return;
 // set input of lineEdit to var 
 QString purpose = HoBIT_String::replaceSpecialChars(lineEditPurpose->text());
 validatePurpose(purpose, _LastInputPurpose);
 updateEditor(lineEditPurpose, _LastInputPurpose);  
 _PurposeLines->at(scrollBarLines->value()) = _LastInputPurpose;                    
}



/** 
*validation of lineEdit Purpose2, 
*opens messagebox if validation is false
*/
void GuiTransferForm::validateInputPurpose2()
{
 if(_IsScrolling) //No validation when scrolling, otherwise an endless loop is created
   return;
 // set input of lineEdit to var 
 QString purpose = HoBIT_String::replaceSpecialChars(lineEditPurpose2->text());
 validatePurpose(purpose, _LastInputPurpose2);
 updateEditor(lineEditPurpose2, _LastInputPurpose2);                      
 _PurposeLines->at(scrollBarLines->value() + 1 ) = _LastInputPurpose2;
 if(_LastInputPurpose2.length() >= 27){
    slotNextLine();
 } 
 else if(lineEditPurpose2->text() == "")
   slotPrevLine();                     
}

/**
*called on reject, emit signal cancelled()
*/
void GuiTransferForm::reject(){
 // Clear Form
 clear();
 emit cancelled();
}
/** Clears the form.
*/
void GuiTransferForm::clear(){
 //clear gui
 lineEditOtherName->setText("");
 lineEditOtherBankCode->setText("");
 lineEditOtherAccountId->setText("");
 lineEditValue->setText("");
 dateEditFirst->setDate(QDate::currentDate());
 dateEditLast->setDate(QDate::currentDate());
 
 //Clearing purpose is tricky, because of triple buffering
 //first of all clear buffers
 _LastInputPurpose= "";
 _LastInputPurpose2= "";
 //clear _PurposeLines and _PurposeList
 for(uint nLine = 0; nLine < 14; nLine++)
   _PurposeLines->at(nLine) = "";
 _PurposeList.clear();  
 scrollBarLines->hide();  
 //finally clear edits. This causes textChanged signals!
 lineEditPurpose->setText("");
 lineEditPurpose2->setText("");
}

/** Validates purpose string. Only DTAUS characters are allowed here.
*/
bool GuiTransferForm::validatePurpose(QString purpose, QString & lastInput){
  // regular expression allows letters, blanks and digits with a length of 0 to 20
 QRegExp rx(tr("[a-z,A-Z,0-9,/,\\.\\,\\+\\-\\s]{0,28}"), "0 to 27 characters legal for a purpose"  );
 QRegExpValidator v( rx, 0 );
 int pos = 0;
 
 //illegal char, display message and keep old string
 if(v.validate(purpose, pos) == QValidator::Acceptable  || purpose.isEmpty()) 
   lastInput = purpose;
 else {
   QMessageBox::warning( this,
                         tr("Error"),
                         tr("Input Error\n This character is not allowed here."),
                         tr("OK"), 
                         QString::null, 0, 1 ) ;
   return false;                      
 }                        
 // too long, display message and truncate
 if(purpose.length() > 27)
 { 
   QMessageBox::warning( this,
                         tr("Error"),
                         tr("Input Error\n If you need more than 27 characters, please use the next line."),
                         tr("OK"), 
                         QString::null, 0, 1 ) ;
   lastInput.truncate(27);  
   return false;                     
 }   
 else return true;              
}

/** Asks the user for reconfirmation. Returns true, if user clicks yes.
*/
int GuiTransferForm::userReconfirmation(){
  QString text = "<table>\n" +  reconfirmationTableRows() + "</table>\n";  
   // messagebox opens for show input of form and ask for confirm, alternatively decline
/*
   int userCommand =
   bool ok = QMessageBox::question( this,
                                    tr("Confirm transfer"),
                                    text,
                                    tr("&Yes"), tr("&No"), "",
                                    1, 1) == 0; //returns 0, if the Yes button (button 0) is clicked
  return ok;
  */
  DlgConfirmTransaction dlg(this, text); 
  dlg.exec();
  return dlg.getUserCommand();                                   
}

/** Builds the html table rows for the reconfirmation dialog. Returns a string that ist to be included into <table> </table tags. 
*/
QString GuiTransferForm::reconfirmationTableRows(){
  QString result;
  QString ourAccount = _hobit->getCurrentAccount();   
  QString name = lineEditOtherName->text();
  QString bankCode = lineEditOtherBankCode->text();
  QString accountId = lineEditOtherAccountId->text();
  QString purpose1 =  lineEditPurpose->text();
  QString purpose2 =  lineEditPurpose2->text();
  QString value = lineEditValue->text();
  // text for show all inputs of form, used in messagebox

  result = "<tr><td colspan =2>" +   _ReconfirmationHeader +"</td></tr>\n";

  result += "<tr>";
  result += "<td>" + tr("From Account:") + "</td>";
  result += "<td>" + ourAccount + " " +_hobit->getCurrentAccountName() + "</td>";
  result += "</tr>\n";

  result += "<tr>";
  result += "<td>" + tr("To Account:")+ "</td>";
  result += "<td>" + accountId +  "</td>";
  result += "</tr>\n";

  result += "<tr>";
  result += "<td>" + tr("At Bank:")+  "</td>";
  result += "<td>" + bankCode + " " + textLabelBankName->text() + "</td>";
  result += "</tr>\n";

  result += "<tr>";
  result += "<td>" + tr("Amount:") +  "</td>";
  result += "<td>" + value +  tr(" EUR")+ "</td>";
  result += "</tr>\n";

  if(updatePurposeList() == 0){
    result += "<tr>";
    result += "<td>" + tr("No Purpose") +  "</td>";
    result += "<td>"  "</td>";
    result += "</tr>\n";
  }  
  else {
    QStringList::iterator it;
    it= _PurposeList.begin();
    QString line = (*it);
    result += "<tr>";
    result += "<td>" + tr("Purpose:") +  "</td>";
    result += "<td>" + line + "</td>";
    result += "</tr>\n";
    it++;
   while ( it!=_PurposeList.end()) {
      line = (*it);
      result += "<tr>";
      result += "<td> </td>"; //leave first column empty
      result += "<td>" + line +  "</td>";      
      result += "</tr>\n";
      it++;
   }
  }//end else 
  return result;
}

/** Checks for empty fields, BLZ for 8 digits and value for !=0.0
*/
bool GuiTransferForm::isOk(){     

 if(lineEditOtherBankCode->text().length() != 8){
   QMessageBox::critical(this, tr("Error"), 
                               tr("The bankcode is not ok. Please correct."));
   return false;
 }

 QString value = lineEditValue->text();
 //replace the decimal komma with a dot for call to toDouble
 for(uint pos = 0; pos < value.length(); pos++)
   if(value[pos] == ',')
     value[pos] = '.';

 if(value.toDouble() == 0.0){
   QMessageBox::critical(this, tr("Error"), 
                               tr("The value to be transfered is zero. Please correct."));
   return false;
 }
  // check if all necessary fields are not empty, otherwise errormessage
 if(lineEditOtherName->text().isEmpty() || lineEditOtherBankCode->text().isEmpty() 
    || lineEditOtherAccountId->text().isEmpty()    || lineEditValue->text().isEmpty()){
    QMessageBox::warning( this,
                          tr("Error"),
                          tr("Please fill in all fields. Purpose fields are not mandatory."),
                          tr("&OK"),
                          QString::null, 0, 1 ) ;
   return false;
 }
 
 if(updatePurposeList() == 0){ 
   if( QMessageBox::question( this,
                          tr("Warning"),
                          tr("No purpose has been given. Do you want to send without purpose?"),
                          QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes)
     return true;
   else return false;                       
 }
 return true; 
}

/** Clears all edits.
*/
void GuiTransferForm::slotClearForm(){
 clear();
}

/** Updates the editor's text and takes care of cursor position in
 * replaced text.
 */
void GuiTransferForm::updateEditor( QLineEdit * edit, const QString & newText){
  int cursorPos = edit->cursorPosition();
  cursorPos += newText.length() - edit->text().length(); //if text is now longer because of replaceSpecialChars, add difference
  edit->setText(newText);
  edit->setCursorPosition(cursorPos);
}

/** Scrolls the purpose lines upwards
*/
void GuiTransferForm::slotNextLine(){
 if(lineEditPurpose->hasFocus()){
   lineEditPurpose2->setFocus();
   return;
 }
 else if(!scrollBarLines->isVisible())
   scrollBarLines->show();
 if(scrollBarLines->value() == scrollBarLines->maxValue()){
   QMessageBox::critical(this, tr("Error"), tr("Sorry, there are no more purpose lines available.")); 
 }  
 else scrollBarLines->setValue(scrollBarLines->value() +1);  
}

/** Scrolls the purpose lines downwards
*/
void GuiTransferForm::slotPrevLine(){
  scrollBarLines->setValue(scrollBarLines->value() -1);   
  if((scrollBarLines->value() > 0) ||((scrollBarLines->value() == 0) && !lineEditPurpose2->text().isEmpty())){ 
    QApplication::postEvent(lineEditPurpose2, new QKeyEvent( QEvent::KeyPress, Qt::Key_E, 'e',  Qt::ControlButton)); //put cursor at the end
    QApplication::postEvent(lineEditPurpose2, new QKeyEvent( QEvent::KeyRelease, Qt::Key_E,'e',  Qt::ControlButton));
  } 
  else if(lineEditPurpose2->text().isEmpty()){
   lineEditPurpose->setFocus();
   lineEditPurpose->setCursorPosition(lineEditPurpose->text().length()); 
  }                      
}

void GuiTransferForm::slotSliderChanged(int value){
 _IsScrolling = true;
 
  lineEditPurpose->setText( _PurposeLines->at(value)); 
  _LastInputPurpose = lineEditPurpose->text();              
  lineEditPurpose2->setText( _PurposeLines->at(value + 1)); 
  _LastInputPurpose2 = lineEditPurpose2->text(); 
 
 if(scrollBarLines->value() ==0){
     //hide scroller, if only two lines are needed  
    bool scrollerRequired = false;  
    for(uint nLine = 2; nLine < _PurposeLines->count(); nLine++)
      if(_PurposeLines->at(nLine) != "")
        scrollerRequired = true;
    if(!scrollerRequired)    
      scrollBarLines->hide();
 }
 _IsScrolling = false;               
}

/** Sets _PurposeLines and purpose edits
*/
void GuiTransferForm::setPurpose(QStringList purpose){
  QStringList::iterator it = purpose.begin();
  for(uint nLine = 0; nLine < _PurposeLines->count(); nLine++){
    if(it != purpose.end()){
      QString line = (*it);
      _PurposeLines->at(nLine) = line;
      it++;
    }
    else _PurposeLines->at(nLine)= "";    
  }//end for
  
  lineEditPurpose->setText(_PurposeLines->at(0));
  lineEditPurpose2->setText(_PurposeLines->at(1));
}

/** updates the purpose list and returns number of purpose lines.
*/
int GuiTransferForm::updatePurposeList(){
 int lines = 0;
 _PurposeList.clear();
 for(uint nLine = 0; nLine < _PurposeLines->count(); nLine ++){
  if(_PurposeLines->at(nLine) != ""){ 
    _PurposeList.push_back(_PurposeLines->at(nLine).ascii());
    lines ++;
  }
 }
 return lines;
}

/** Creates the job and reurns true on success. To be overwritten.
  */
bool GuiTransferForm::createJob(){
    qDebug("Error: GuiTransferForm::createJob() was called.");
    return false;
}   
