/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef KEYFILERDH2_H
#define KEYFILERDH2_H

#include "keyfile.h"

typedef struct{
  short int version; //2 bytes
  unsigned char  salt [20];
  int   iterations;  //4 bytes 
}RDH2DiskHeader;

typedef struct{
 union{
   char rawData[26];
   RDH2DiskHeader header;
 };
}RDH2DiskHeaderEntry;
/**
This class represents a RDH2 formatted key file. It is used for file format conversions.

@author Walter Roth
*/
class KeyFileRDH2 : public KeyFile
{
public:
    /**Constructor loads data into QByteArray _Data
    */
    KeyFileRDH2(const QString & fileName, const QString & password);

    /** destructor
    */
    ~KeyFileRDH2();

private:
    /** Read and parse the key file. Returns true, if ok. Sends error messages to stderr.
    */
    bool parseFile();
    
    /** The CryptData object used for decrypting the private keys.
    */
    CryptData * _CryptData;
    /** Flag for correctly parsed file.
    */
    bool _Ok; 
       
    /** The file header
    */
    RDH2DiskHeader _Header;
    
    HoBIT_TLV _TLV_Header;
};

#endif
