/***************************************************************************
                          transactionitem.cpp  -  description
                             -------------------
    begin                : Fr, march 24th 2006
    copyright            : (C) 2006 by Walter Roth
    email                : moneypenny@in-fh-swf.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
#include "transactionitem.h"
#include <openhbci2/date.h>
#include <qstring.h>
#include <qdatetime.h>
#include <qlistview.h>
#include <string>
#include <qlabel.h>

 
 TransactionItem::TransactionItem(Pointer<Transaction> t){
   _Transaction = t;
 }   

TransactionItem::~TransactionItem(){
 //do not delete table items, they are owned by table
}

/** Read property of DatedTransfer _datedTransfer. */
const Pointer<Transaction> TransactionItem::transaction(){
    return _Transaction;
}


/** Encodes the Transaction data as an html- table row string.
*/
QString TransactionItem::htmlTableRow(const QString & bgColor){
  QString result;
  Date d = _Transaction.ref().date();
  QDate date(d.year(), d.month(), d.day());
  QString value = QString::number(_Transaction.ref().value().getValue(),'f',2); // 2 digits after . "-999.99"  
  QString ouraccId = _Transaction.ref().ouraccountId().c_str();
  QString ourbankCode = _Transaction.ref().ourbankcode().c_str();
  QString otheraccId = _Transaction.ref().otheraccountId().c_str();
  QString otherbankCode = _Transaction.ref().otherbankcode().c_str();
  QString othername = _Transaction.ref().othername().c_str();
  QString custref = _Transaction.ref().custref().c_str();
  QString key = _Transaction.ref().key().c_str();
  QString primanota = _Transaction.ref().primanota().c_str();
  
  list<string>::iterator line;
  list<string>lines = _Transaction.ref().purposeLines();
  QString purposeText;
  for(line = lines.begin(); line!= lines.end(); line++) {//chain the lines together
   if(!(*line).empty())
     purposeText += QString((*line).c_str()) + "<br>";
  } 
  purposeText.truncate(purposeText.length() -4); //delete last <br>
  
  string text = _Transaction.ref().text();
  
   result += "<tr bgcolor =\"" + bgColor + "\">";
   
   result += "<td>";
    result += date.toString( Qt::LocalDate);
   result += "</td>";
   
   result += "<td>";
    result += othername;
   result += "</td>";
   
   result += "<td>";
    result += purposeText;
   result += "</td>";
   
   result += "<td>";
    result += text;
   result += "</td>";
   
   result += "<td>";
    result += otherbankCode;
   result += "</td>";
   
   result += "<td>";
    result += otheraccId;
   result += "</td>";  
  
   result += "<td align =\"right\">";
    result += value;
   result += "</td>";
   
 result += "</tr>\n";
 
 return result;
}


/** Read property of QString _OurAccount. 
*/
QString TransactionItem::getOurAccountId()const{
    return QString(_Transaction.ref().ouraccountId().c_str());
}

/** Read property of QString _OtherAccount. 
*/
QString TransactionItem::getOtherAccountId()const{
    return QString(_Transaction.ref().otheraccountId().c_str());
}

/** Read property of QString _OtherBankCode. 
*/
QString TransactionItem::getOtherBankCode()const{
    return QString(_Transaction.ref().otherbankcode().c_str());
}

/** Read property of QString _OurBankCode. 
*/
QString TransactionItem::getOurBankCode()const{
    return QString(_Transaction.ref().ourbankcode().c_str());
}

/** Read property of QString _OtherName. 
*/
QString TransactionItem::getOtherName()const{
    return QString(_Transaction.ref().othername().c_str());
}
/** Read property of QString _Text. 
*/
QString TransactionItem::getText()const{
    return QString(_Transaction.ref().text().c_str());
}

/** Read property of QString _ValutaDate. 
*/
QDate TransactionItem::getValutaDate()const{
        Date d = _Transaction.ref().valutaDate();
        QDate qD = QDate(d.year(), d.month(), d.day());
    return qD;
}


/** Read property of QString _PurposeLine. First line is line 1
*/
QString TransactionItem::getPurposeLine(int nLine)const{
  list<string>lines = _Transaction.ref().purposeLines();
  list<string>::iterator line;
  QString purposeText;
  int nline = -1;
  for(line = lines.begin(); line!= lines.end() || nline >= nLine; line++) {
     nline++;
     if(nline == nLine)
        return QString((*line).c_str());
  }
  return ""; //just to be shure
}
/** Read property of numeric value  
*/
double TransactionItem::getNumericValue(){
 return _Transaction.ref().value().getValue();
}
/** Read property of QString Purpose.
*/
QStringList TransactionItem::getPurpose()const{
  list<string>lines = _Transaction.ref().purposeLines();
  QStringList resultList;
  list<string>::iterator line;
  for(line = lines.begin(); line!= lines.end(); line ++) {
    if(! (*line).empty())
      resultList.push_back(QString((*line).c_str()));
  }  
  return resultList;  
}

/** Adds the transactionitem as last row to table.
   */
 void TransactionItem::addToTable(QTable * table, int row, const QString & dateFormat){
 QTableItem *trDate;
  QTableItem *otherName;
  //QTableItem *custRef;
  QTableItem *bankCode;
  QTableItem *accountId;
  //QTableItem *textKey;
  //QTableItem *key;
  QTableItem *purpose;
  QTableItem *trText;
  QLabel *  labelValue;  //label allows to set the text color

  QString sValue = QString::number(getNumericValue(),'f',2) + "  "; // 2 digits after . "-999.99"
  if(table->numRows() < row +1)
   table->setNumRows(row +1);
   
  trDate = new QTableItem(table, QTableItem::Never, getValutaDate().toString(dateFormat) );
  trDate->setWordWrap(false);
  table->setItem( row, 0, trDate);
  
  otherName = new QTableItem(table, QTableItem::Never, getOtherName() );
  otherName->setWordWrap(true);
  table->setItem( row, 1, otherName);
   
  labelValue = new QLabel(sValue, table);
  labelValue->setAlignment(Qt::AlignRight); //set alignment here, but set color in paintEvent!
  table->setCellWidget( row, 2, labelValue);
  
  QString purposeText = getPurpose().join("\n");
  purpose = new QTableItem(table, QTableItem::Never, purposeText );
  purpose->setWordWrap(true);
  table->setItem( row, 3, purpose);
 
  bankCode = new QTableItem(table, QTableItem::Never, getOtherBankCode() );
  bankCode->setWordWrap(true);
  table->setItem( row, 4, bankCode);
  
  accountId = new QTableItem(table, QTableItem::Never, getOtherAccountId());
  accountId->setWordWrap(true);
  table->setItem( row, 5, accountId);
  
  trText = new QTableItem(table, QTableItem::Never, getText() );
  trText->setWordWrap(true);
  table->setItem( row, 6, trText);

 /* _CustRef = new QTableItem(table, QTableItem::Never, t.ref().custref().c_str() );
  _CustRef->setWordWrap(true);
  table->setItem( row, 1, _CustRef);
 */ 
  table->adjustRow(row);
    }
 
 